"""
Low-level UUID implementation for Python.

Exposes UUID (compatible with stdlib uuid.UUID), uuid4(), and randstr_16().
"""

from typing import Tuple

from .__about__ import __version__
from .picoid import UUID, randstr_16, uuid4

__all__: Tuple[str, ...] = ("UUID", "__version__", "randstr_16", "uuid4")



cdef extern from "uuid.h":
    cdef void c_uuid4(unsigned char* uuid)

################################################################################
# Copyright (C) 2016-present the asyncpg authors and contributors
# <see AUTHORS file>
#
# This module is part of asyncpg and is released under
# the Apache 2.0 License: http://www.apache.org/licenses/LICENSE-2.0

cdef extern from "hex.h":
    cdef void uuid_to_str(const char *source, char *dest)
    cdef void uuid_to_hex(const char *source, char *dest)
    # cdef void uuid_to_int(const char *source, unsigned long long *dest)

################################################################################



# cython: language_level=3

cimport cpython
cimport cython
from libc.stdint cimport int8_t, uint8_t
from libc.string cimport memcmp, memcpy


cdef extern from "Python.h":
    int PyUnicode_1BYTE_KIND
    const char* PyUnicode_AsUTF8AndSize(object unicode, Py_ssize_t *size) except NULL
    object PyUnicode_FromKindAndData(int kind, const void *buffer, Py_ssize_t size)
    object PyBytes_FromStringAndSize(const char *s, Py_ssize_t len)

cdef extern from "hex.h":
    cdef void uuid_to_str(const char *source, char *dest)
    cdef void uuid_to_hex(const char *source, char *dest)

cdef extern from "uuid.h":
    cdef void c_uuid4(unsigned char* dest) nogil


cdef class __UUIDReplaceMe:
    pass

@cython.final
@cython.no_gc_clear
cdef class UUID(__UUIDReplaceMe):
    cdef char[16] _data
    cdef object _int
    cdef object _hash


cdef UUID uuid_from_buf(const char *buf)
cdef void uuid_bytes_from_str(str u, char *out)

cpdef bytes randstr_16()
cpdef UUID uuid4()



"""Minimal pytest tests for picoid: every exported name works as expected."""

import pytest

import picoid


def test_all_exported_names_are_available() -> None:
    """Every name in __all__ is importable from picoid and is usable."""
    assert hasattr(picoid, "__all__")
    assert isinstance(picoid.__all__, tuple)
    assert all(isinstance(name, str) for name in picoid.__all__)
    for name in picoid.__all__:
        assert hasattr(
            picoid, name
        ), f"picoid.__all__ contains {name!r} but picoid has no attribute {name!r}"


def test_version() -> None:
    """__version__ is a non-empty string."""
    assert hasattr(picoid, "__version__")
    v = picoid.__version__
    assert isinstance(v, str)
    assert len(v) > 0


def test_import_from_picoid() -> None:
    """All __all__ names can be imported via 'from picoid import ...'."""
    for name in picoid.__all__:
        module = __import__("picoid", fromlist=[name])
        assert hasattr(module, name), f"from picoid import {name!r} failed"



import uuid
from typing import Any, Tuple, Union

class UUID(uuid.UUID):
    """
    UUID type compatible with stdlib uuid.UUID, with fast C-backed storage.

    Args:
        - inp: A hyphenated UUID string (32 hex chars with optional hyphens)
              or exactly 16 bytes.

    Returns:
        None (constructor).
    """

    def __init__(self, inp: Union[str, bytes]) -> None: ...
    @property
    def bytes(self) -> bytes: ...
    @property
    def int(self) -> int: ...
    @property
    def is_safe(self) -> uuid.SafeUUID: ...
    def __str__(self) -> str: ...
    @property
    def hex(self) -> str: ...
    def __repr__(self) -> str: ...
    def __reduce__(self) -> Tuple[type, Tuple[bytes]]: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __lt__(self, other: Any) -> bool: ...
    def __gt__(self, other: Any) -> bool: ...
    def __le__(self, other: Any) -> bool: ...
    def __ge__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    def __int__(self) -> int: ...
    @property
    def bytes_le(self) -> bytes: ...
    @property
    def fields(self) -> Tuple[int, int, int, int, int, int]: ...
    @property
    def time_low(self) -> int: ...
    @property
    def time_mid(self) -> int: ...
    @property
    def time_hi_version(self) -> int: ...
    @property
    def clock_seq_hi_variant(self) -> int: ...
    @property
    def clock_seq_low(self) -> int: ...
    @property
    def time(self) -> int: ...
    @property
    def clock_seq(self) -> int: ...
    @property
    def node(self) -> int: ...
    @property
    def urn(self) -> str: ...
    @property
    def variant(self) -> uuid.UUID: ...
    @property
    def version(self) -> int: ...

def randstr_16() -> bytes:
    """
    Return 16 random bytes with UUID4 version/variant bits set.

    Args:
        (none)

    Returns:
        16 random bytes with UUID4 version/variant bits set (bytes).
    """
    ...

def uuid4() -> UUID:
    """
    Generate a random UUID (version 4).

    Args:
        (none)

    Returns:
        A new random UUID4 instance (UUID).
    """
    ...
